@@ -1,10 +1,10 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: help.c 245 2006-11-18 02:46:41Z hubert@u.washington.edu $";
+static char rcsid[] = "$Id: help.c 380 2007-01-23 00:09:18Z hubert@u.washington.edu $";
 #endif
 
 /*
  * ========================================================================
- * Copyright 2006 University of Washington
+ * Copyright 2006-2007 University of Washington
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -80,7 +80,7 @@ static char att_cur_msg[] = "\
 /*
  * Internal prototypes
  */
-int	 helper_internal(HelpType, char **, char *, char *, int);
+int	 helper_internal(HelpType, char *, char *, int);
 int	 help_processor(int, MSGNO_S *, SCROLL_S *);
 void	 help_keymenu_tweek(SCROLL_S *, int);
 void	 print_help(char **);
@@ -90,11 +90,6 @@ int	 print_help_page_break(long, char *, LT_INS_S **, void *);
 int	 help_bogus_input(UCS);
 void	 review_messages(void);
 int	 journal_processor(int, MSGNO_S *, SCROLL_S *);
-int	 gripe_newbody(struct pine *, BODY **, long, int);
-ADDRESS *gripe_token_addr(char *);
-char	*gripe_id(char *);
-int	 gripe_bogus_input(int);
-void	 att_cur_drawer(void);
 int	 help_popup(SCROLL_S *, int);
 #ifdef	_WINDOWS
 int	 help_subsection_popup(SCROLL_S *, int);
@@ -108,9 +103,6 @@ int	 help_subsection_popup(SCROLL_S *, int);
     Args: text   -- The help text to display (from pine.help --> helptext.c)
           title  -- The title of the help text 
 
-	  The dumb otext argument is for when we want to pass in a char **
-	  data item no matter what the type of HelpType is.
-  
   Result: format text and call scroller
 
   The pages array contains the line number of the start of the pages in
@@ -119,7 +111,7 @@ The list is ended with a page line number of -1. Line number 0 is also
 the first line in the text.
   -----*/
 int
-helper_internal(HelpType text, char **otext, char *frag, char *title, int flags)
+helper_internal(HelpType text, char *frag, char *title, int flags)
 {
     char	  **shown_text;
     int		    cmd = MC_NONE;
@@ -129,23 +121,11 @@ helper_internal(HelpType text, char **otext, char *frag, char *title, int flags)
     HANDLE_S	   *handles = NULL, *htmp;
     HELP_SCROLL_S   hscroll;
     gf_io_t	    pc;
-#ifdef	HELPFILE
-    char	  **dynamic_text = NULL;
-#endif /* HELPFILE */
 
     dprint((1, "\n\n    ---- HELPER ----\n"));
 
-#ifdef	HELPFILE
-    if(otext)
-      shown_text = otext;
-    else if((shown_text = dynamic_text = get_help_text(text)) == NULL)
-      return(cmd);
-#else
-    if(otext)
-      shown_text = otext;
-    else
-      shown_text = text;
-#endif /* HELPFILE */
+    /* assumption here is that HelpType is char **  */
+    shown_text = text;
 
     if(F_ON(F_BLANK_KEYMENU,ps_global)){
 	FOOTER_ROWS(ps_global) = 3;
@@ -349,11 +329,6 @@ helper_internal(HelpType text, char **otext, char *frag, char *title, int flags)
     }
     while(cmd == MC_RESIZE);
 
-#ifdef	HELPFILE
-    if(dynamic_text)
-      free_list_array(&dynamic_text);
-#endif
-
     return(cmd);
 }
 
@@ -364,7 +339,7 @@ helper_internal(HelpType text, char **otext, char *frag, char *title, int flags)
 int
 helper(HelpType text, char *title, int flags)
 {
-    return(helper_internal(text, NULL, NULL, title, flags));
+    return(helper_internal(text, NULL, title, flags));
 }
 
 
@@ -515,24 +490,14 @@ print_help(char **text)
 void
 print_all_help(void)
 {
-#ifdef	HELPFILE
-    short t;
-#else
     struct help_texts *t;
-#endif
     char **h;
 
     if(open_printer(_("all 150+ pages of help text")) == 0) {
 #ifndef	DOS
 	intr_handling_on();
 #endif
-#ifdef	HELPFILE
-        for(t = 0; t < LASTHELP; t++) {
-	    if(!(h = get_help_text(t)))
-	      return;
-#else
 	for(t = h_texts; (h = t->help_text) != NO_HELP; t++) {
-#endif
 	    if(ps_global->intr_pending){
 		q_status_message(SM_ORDER, 3, 3,
 				 _("Print of all help cancelled"));
@@ -540,10 +505,6 @@ print_all_help(void)
 	    }
 
 	    print_help(h);
-
-#ifdef	HELPFILE
-	    free_list_array(&h);
-#endif
         }
 
 #ifndef	DOS
@@ -560,7 +521,7 @@ print_all_help(void)
 void
 print_help_page_title(char *buf, size_t buflen, HPRT_S *hprt)
 {
-    snprintf(buf, buflen, "  Pine Help%s%.*s%*s%d",
+    snprintf(buf, buflen, "  Alpine Help%s%.*s%*s%d",
 	    hprt->title_len ? ": " : " Text",
 	    MIN(55, hprt->title_len), hprt->title_len ? hprt->title : "",
 	    59 - (hprt->title_len ? MIN(55, hprt->title_len) : 5),
@@ -629,7 +590,7 @@ url_local_helper(char *url)
 	if(newhelp != NO_HELP){
 	    int rv;
 
-	    rv = helper_internal(newhelp, NULL, frag, _("HELP SUB-SECTION"),
+	    rv = helper_internal(newhelp, frag, _("HELP SUB-SECTION"),
 				 HLPD_NEWWIN | HLPD_SIMPLE | HLPD_FROMHELP);
 	    ps_global->mangled_screen = 1;
 	    return((rv == MC_EXIT) ? 2 : 1);
@@ -652,7 +613,7 @@ url_local_config(char *url)
 	config = get_supported_options();
 	if(config){
 	    /* TRANSLATORS: Help for configuration */
-	    rv = helper_internal(NO_HELP, config, NULL, _("HELP CONFIG"),
+	    rv = helper_internal(config, NULL, _("HELP CONFIG"),
 				 HLPD_NEWWIN | HLPD_SIMPLE | HLPD_FROMHELP);
 	    free_list_array(&config);
 	}
@@ -935,432 +896,21 @@ journal_processor(int cmd, MSGNO_S *msgmap, SCROLL_S *sparms)
 #endif
 
 
-#ifdef notdef
-/*
- * I don't think this is used.
- */
-int
-gripe_gripe_to(url)
-    char *url;
-{
-    char      *composer_title, *url_copy, *optstr, *p;
-    int	       opts = 0;
-    BODY      *body = NULL;
-    ENVELOPE  *outgoing = NULL;
-    REPLY_S    fake_reply;
-    PINEFIELD *pf = NULL;
-    long       msgno = mn_m2raw(ps_global->msgmap, 
-				mn_get_cur(ps_global->msgmap));
-
-    url_copy = cpystr(url + 13);
-    if(optstr = strchr(url_copy, '?'))
-      *optstr++ = '\0';
-
-    outgoing		 = mail_newenvelope();
-    outgoing->message_id = generate_message_id();
-
-    if(outgoing->to = gripe_token_addr(url_copy)){
-	composer_title = _("COMPOSE TO LOCAL SUPPORT");
-	dprint((1, 
-		   "\n\n   -- Send to local support(%s@%s) --\n",
-		   outgoing->to->mailbox ? outgoing->to->mailbox : "NULL",
-		   outgoing->to->host ? outgoing->to->host : "NULL"));
-    }
-    else{			/* must be global */
-	composer_title = _("REQUEST FOR ASSISTANCE");
-	rfc822_parse_adrlist(&outgoing->to, url_copy, ps_global->maildomain);
-    }
-
-    /*
-     * Sniff thru options
-     */
-    while(optstr){
-	if(p = strchr(optstr, '?'))	/* tie off list item */
-	  *p++ = '\0';
-
-	if(!strucmp(optstr, "config"))
-	  opts |= GRIPE_OPT_CONF;
-	else if(!strucmp(optstr, "curmsg"))
-	  opts |= GRIPE_OPT_MSG;
-	else if(!strucmp(optstr, "local"))
-	  opts |= GRIPE_OPT_LOCAL;
-	else if(!strucmp(optstr, "keys"))
-	  opts |= GRIPE_OPT_KEYS;
-
-	optstr = p;
-    }
-
-    /* build body and hand off to composer... */
-    if(gripe_newbody(ps_global, &body, msgno, opts) == 0){
-	pf = (PINEFIELD *) fs_get(sizeof(PINEFIELD));
-	memset(pf, 0, sizeof(PINEFIELD));
-	pf->name		   = cpystr("X-Generated-Via");
-	pf->type		   = FreeText;
-	pf->textbuf		   = gripe_id("Pine Bug Report screen");
-	memset((void *)&fake_reply, 0, sizeof(fake_reply));
-	fake_reply.flags	   = REPLY_PSEUDO;
-	fake_reply.data.pico_flags = P_HEADEND;
-	pine_send(outgoing, &body, composer_title, NULL, NULL,
-		  &fake_reply, NULL, NULL, pf, 0);
-    }
-    
-    ps_global->mangled_screen = 1;
-    mail_free_envelope(&outgoing);
-
-    if(body)
-      pine_free_body(&body);
-
-    fs_give((void **) &url_copy);
-    
-    return(10);
-}
-
-
-int
-gripe_newbody(ps, body, msgno, flags)
-    struct pine *ps;
-    BODY       **body;
-    long         msgno;
-    int          flags;
-{
-    BODY        *pb;
-    PART       **pp;
-    STORE_S	*store;
-    gf_io_t      pc;
-    static char *err = "Problem creating space for message text.";
-    int          i;
-    char         tmp[MAILTMPLEN], *p;
-
-    if(store = so_get(PicoText, NULL, EDIT_ACCESS)){
-	*body = mail_newbody();
-
-	if((p = detoken(NULL, NULL, 2, 0, 1, NULL, NULL)) != NULL){
-	    if(*p)
-	      so_puts(store, p);
-
-	    fs_give((void **) &p);
-	}
-    }
-    else{
-	q_status_message(SM_ORDER | SM_DING, 3, 4, err);
-	return(-1);
-    }
-
-    if(flags){
-	/*---- Might have multiple parts ----*/
-	(*body)->type			= TYPEMULTIPART;
-	/*---- The TEXT part/body ----*/
-	(*body)->nested.part            = mail_newbody_part();
-	(*body)->nested.part->body.type = TYPETEXT;
-	(*body)->nested.part->body.contents.text.data = (void *) store;
-
-	/*---- create object, and write current config into it ----*/
-	pp = &((*body)->nested.part->next);
-
-	if(flags & GRIPE_OPT_CONF){
-	    *pp			     = mail_newbody_part();
-	    pb			     = &((*pp)->body);
-	    pp			     = &((*pp)->next);
-	    pb->type		     = TYPETEXT;
-	    pb->id		     = generate_message_id();
-	    pb->description	     = cpystr("Pine Configuration Data");
-	    pb->parameter	     = mail_newbody_parameter();
-	    pb->parameter->attribute = cpystr("name");
-	    pb->parameter->value     = cpystr("config.txt");
-
-	    if(store = so_get(CharStar, NULL, EDIT_ACCESS)){
-		extern char datestamp[], hoststamp[];
-
-		pb->contents.text.data = (void *) store;
-		gf_set_so_writec(&pc, store);
-		gf_puts("Pine built ", pc);
-		gf_puts(datestamp, pc);
-		gf_puts(" on host: ", pc);
-		gf_puts(hoststamp, pc);
-		gf_puts("\n", pc);
-
-		dump_pine_struct(ps, pc);
-		dump_config(ps, pc, 0);
-
-		pb->size.bytes = strlen((char *) so_text(store));
-		gf_clear_so_writec(store);
-	    }
-	    else{
-		q_status_message(SM_ORDER | SM_DING, 3, 4, err);
-		return(-1);
-	    }
-	}
-
-	if(flags & GRIPE_OPT_KEYS){
-	    *pp			     = mail_newbody_part();
-	    pb			     = &((*pp)->body);
-	    pp			     = &((*pp)->next);
-	    pb->type		     = TYPETEXT;
-	    pb->id		     = generate_message_id();
-	    pb->description	     = cpystr("Recent User Input");
-	    pb->parameter	      = mail_newbody_parameter();
-	    pb->parameter->attribute  = cpystr("name");
-	    pb->parameter->value      = cpystr("uinput.txt");
-
-	    if(store = so_get(CharStar, NULL, EDIT_ACCESS)){
-		pb->contents.text.data = (void *) store;
-
-		so_puts(store, "User's most recent input:\n");
-
-		/* dump last n keystrokes */
-		so_puts(store, "========== Latest keystrokes ==========\n");
-		while((i = key_playback(0)) != -1){
-		    snprintf(tmp, sizeof(tmp), "\t%s\t(0x%x)\n", pretty_command(i), i);
-		    tmp[sizeof(tmp)-1] = '\0';
-		    so_puts(store, tmp);
-		}
-
-		pb->size.bytes = strlen((char *) so_text(store));
-	    }
-	    else{
-		q_status_message(SM_ORDER | SM_DING, 3, 4, err);
-		return(-1);
-	    }
-	}
-
-	/* check for local debugging info? */
-	if((flags & GRIPE_OPT_LOCAL)
-	   && ps_global->VAR_BUGS_EXTRAS 
-	   && can_access(ps_global->VAR_BUGS_EXTRAS, EXECUTE_ACCESS) == 0){
-	    char *error		      = NULL;
-
-	    *pp			      = mail_newbody_part();
-	    pb			      = &((*pp)->body);
-	    pp			      = &((*pp)->next);
-	    pb->type		      = TYPETEXT;
-	    pb->id		      = generate_message_id();
-	    pb->description	      = cpystr("Local Configuration Data");
-	    pb->parameter	      = mail_newbody_parameter();
-	    pb->parameter->attribute  = cpystr("name");
-	    pb->parameter->value      = cpystr("lconfig.txt");
-
-	    if(store = so_get(CharStar, NULL, EDIT_ACCESS)){
-		PIPE_S  *syspipe;		
-		gf_io_t  gc;
-		
-		pb->contents.text.data = (void *) store;
-		gf_set_so_writec(&pc, store);
-		if(syspipe = open_system_pipe(ps_global->VAR_BUGS_EXTRAS,
-					 NULL, NULL,
-					 PIPE_READ | PIPE_STDERR | PIPE_USER,
-					 0, pipe_callback, pipe_report_error)){
-		    gf_set_readc(&gc, (void *)syspipe, 0, PipeStar);
-		    gf_filter_init();
-		    error = gf_pipe(gc, pc);
-		    (void) close_system_pipe(&syspipe, NULL, pipe_callback);
-		}
-		else
-		  error = "executing config collector";
-
-		gf_clear_so_writec(store);
-	    }
-	    
-	    if(error){
-		q_status_message1(SM_ORDER | SM_DING, 3, 4, 
-				  "Problem %s", error);
-		return(-1);
-	    }
-	    else			/* fixup attachment's size */
-	      pb->size.bytes = strlen((char *) so_text(store));
-	}
-
-	if((flags & GRIPE_OPT_MSG) && mn_get_total(ps->msgmap) > 0L){
-	    int ch = 0;
-	
-	    ps->redrawer = att_cur_drawer;
-	    att_cur_drawer();
-
-	    if((ch = one_try_want_to("Attach current message to report",
-				     'y','x',NO_HELP,
-				     WT_FLUSH_IN|WT_SEQ_SENSITIVE)) == 'y'){
-		*pp		      = mail_newbody_part();
-		pb		      = &((*pp)->body);
-		pp		      = &((*pp)->next);
-		pb->type	      = TYPEMESSAGE;
-		pb->id		      = generate_message_id();
-		snprintf(tmp, sizeof(tmp), "Problem Message (%ld of %ld)",
-			mn_get_cur(ps->msgmap), mn_get_total(ps->msgmap));
-		tmp[sizeof(tmp)-1] = '\0';
-		pb->description	      = cpystr(tmp);
-
-		/*---- Package each message in a storage object ----*/
-		if(store = so_get(PART_SO_TYPE, NULL, EDIT_ACCESS)){
-		    pb->contents.text.data = (void *) store;
-		}
-		else{
-		    q_status_message(SM_ORDER | SM_DING, 3, 4, err);
-		    return(-1);
-		}
-
-		/* write the header */
-		if((p = mail_fetch_header(ps->mail_stream, msgno, NIL, NIL,
-					  NIL, FT_PEEK)) && *p)
-		  so_puts(store, p);
-		else
-		  return(-1);
-
-#if	defined(DOS) && !defined(WIN32)
-		/* write fetched text to disk */
-		mail_parameters(ps->mail_stream, SET_GETS, (void *)dos_gets);
-		append_file = (FILE *) so_text(store);
-
-		/* HACK!  See mailview.c:format_message for details... */
-		ps->mail_stream->text = NULL;
-		/* write the body */
-		if(!pine_mail_fetch_text(ps->mail_stream, msgno,
-					 NULL, NULL, NIL))
-		  return(-1);
-
-		pb->size.bytes = ftell(append_file);
-		/* next time body may stay in core */
-		mail_parameters(ps->mail_stream, SET_GETS, (void *)NULL);
-		append_file   = NULL;
-		mail_gc(ps->mail_stream, GC_TEXTS);
-		so_release(store);
-#else
-		pb->size.bytes = strlen(p);
-		so_puts(store, "\015\012");
-
-		if((p = pine_mail_fetch_text(ps->mail_stream, 
-					     msgno, NULL, NULL, NIL))
-		   &&  *p)
-		  so_puts(store, p);
-		else
-		  return(-1);
-
-		pb->size.bytes += strlen(p);
-#endif
-	    }
-	    else if(ch == 'x'){
-		q_status_message(SM_ORDER, 0, 3, "Bug report cancelled.");
-		return(-1);
-	    }
-	}
-    }
-    else{
-	/*---- Only one part! ----*/
-	(*body)->type = TYPETEXT;
-	(*body)->contents.text.data = (void *) store;
-    }
-
-    return(0);
-}
-
-
-ADDRESS *
-gripe_token_addr(token)
-    char *token;
-{
-    char    *p;
-    ADDRESS *a = NULL;
-
-    if(token && *token++ == '_'){
-	if(!strcmp(token, "LOCAL_ADDRESS_")){
-	    p = (ps_global->VAR_LOCAL_ADDRESS
-		 && ps_global->VAR_LOCAL_ADDRESS[0])
-		    ? ps_global->VAR_LOCAL_ADDRESS
-		    : "postmaster";
-	    a = rfc822_parse_mailbox(&p, ps_global->maildomain);
-	    a->personal = cpystr((ps_global->VAR_LOCAL_FULLNAME 
-				  && ps_global->VAR_LOCAL_FULLNAME[0])
-				    ? ps_global->VAR_LOCAL_FULLNAME 
-				    : "Place to report Pine Bugs");
-	}
-	else if(!strcmp(token, "BUGS_ADDRESS_")){
-	    p = (ps_global->VAR_BUGS_ADDRESS
-		 && ps_global->VAR_BUGS_ADDRESS[0])
-		    ? ps_global->VAR_BUGS_ADDRESS : "postmaster";
-	    a = rfc822_parse_mailbox(&p, ps_global->maildomain);
-	    a->personal = cpystr((ps_global->VAR_BUGS_FULLNAME 
-				  && ps_global->VAR_BUGS_FULLNAME[0])
-				    ? ps_global->VAR_BUGS_FULLNAME 
-				    : "Place to report Pine Bugs");
-	}
-    }
-
-    return(a);
-}
-
-
-char *
-gripe_id(key)
-    char *key;
-{
-    int i;
-    
-    /*
-     * Build our contribution to the subject; part constant string
-     * and random 4 character alpha numeric string.
-     */
-    tmp_20k_buf[0] = '\0';
-    snprintf(tmp_20k_buf, SIZEOF_20KBUF, "%s (ID %c%c%d%c%c)", key,
-	    ((i = (int)(random() % 36L)) < 10) ? '0' + i : 'A' + (i - 10),
-	    ((i = (int)(random() % 36L)) < 10) ? '0' + i : 'A' + (i - 10),
-	    (int)(random() % 10L),
-	    ((i = (int)(random() % 36L)) < 10) ? '0' + i : 'A' + (i - 10),
-	    ((i = (int)(random() % 36L)) < 10) ? '0' + i : 'A' + (i - 10));
-    tmp_20k_buf[SIZEOF_20KBUF-1] = '\0';
-    return(cpystr(tmp_20k_buf));
-}
-
-
-int
-gripe_bogus_input(ch)
-    int ch;
-{
-    return(1);			/* done! */
-}
-#endif /* notdef */
-
-
-/*
- * Used by gripe_tool.
- */
-void
-att_cur_drawer(void)
-{
-    int	       i, dline, j;
-    char       buf[256+1];
-
-    /* blat helpful message to screen */
-    ClearBody();
-    j = 0;
-    for(dline = 2;
-	dline < ps_global->ttyo->screen_rows - FOOTER_ROWS(ps_global);
-	dline++){
-	for(i = 0; i < 256 && att_cur_msg[j] && att_cur_msg[j] != '\n'; i++)
-	  buf[i] = att_cur_msg[j++];
-
-	buf[i] = '\0';
-	if(att_cur_msg[j])
-	  j++;
-	else if(!i)
-	  break;
-
-        PutLine0(dline, 1, buf);
-    }
-}
-
-
 #ifdef	_WINDOWS
 
 /*
  * 
  */
 char *
-pcpine_help(section)
-    HelpType section;
+pcpine_help(HelpType section)
 {
     char    **help_lines, *help_text = NULL;
     STORE_S  *store;
     gf_io_t   pc;
 
-    if ((help_lines = get_help_text (section)) != NULL){
+    /* assumption here is that HelpType is char **  */
+    help_lines = section;
+    if (help_lines != NULL){
 	init_helper_getc(help_lines);
 	if(store = so_get(CharStar, NULL, EDIT_ACCESS)){
 	    gf_set_so_writec(&pc, store);
@@ -1381,8 +931,6 @@ pcpine_help(section)
 	    gf_clear_so_writec(store);
 	    so_give(&store);
 	}
-
-	free_list_array(&help_lines);
     }
 
     return(help_text);
@@ -1393,9 +941,7 @@ pcpine_help(section)
  * 
  */
 int
-help_popup(sparms, in_handle)
-    SCROLL_S *sparms;
-    int	      in_handle;
+help_popup(SCROLL_S *sparms, int in_handle)
 {
     MPopup hp_menu[10];
     int	   i = -1;
@@ -1424,9 +970,7 @@ help_popup(sparms, in_handle)
  * 
  */
 int
-help_subsection_popup(sparms, in_handle)
-    SCROLL_S *sparms;
-    int	      in_handle;
+help_subsection_popup(SCROLL_S *sparms, int in_handle)
 {
     MPopup hp_menu[10];
     int	   i = -1;
