@@ -2,7 +2,7 @@
  * $Id: termin.unx.h 136 2006-09-22 20:06:05Z hubert@u.washington.edu $
  *
  * ========================================================================
- * Copyright 2006 University of Washington
+ * Copyright 2006-2007 University of Washington
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -18,6 +18,8 @@
 
 
 #include <general.h>
+#include "../radio.h"
+#include "../../pith/helptext.h"
 
 
 /* exported prototypes */
@@ -30,6 +32,7 @@ void		init_keyboard(int);
 void		end_keyboard(int);
 int		pre_screen_config_opt_enter(char *, int, char *,
 					    ESCKEY_S *, HelpType, int *);
+int             win_dialog_opt_enter(char *, int, char *, ESCKEY_S *, HelpType, int *);
 void            intr_proc(int);
 UCS             extended_code(unsigned kc);
 
