@@ -1,9 +1,9 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: signal.c 223 2006-11-07 01:14:55Z jpf@u.washington.edu $";
+static char rcsid[] = "$Id: signal.c 380 2007-01-23 00:09:18Z hubert@u.washington.edu $";
 #endif
 
 /* ========================================================================
- * Copyright 2006 University of Washington
+ * Copyright 2006-2007 University of Washington
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -261,12 +261,12 @@ hup_signal(void)
 void
 user_input_timeout_exit(int to_hours)
 {
-    char msg[40];
+    char msg[80];
 
     dprint((1,
 	   "\n\n** Exiting: user input timeout (%d hours) **\n\n\n\n",
 	   to_hours));
-    snprintf(msg, sizeof(msg), _("\n\nPine timed out (No user input for %d %s)\n"), to_hours,
+    snprintf(msg, sizeof(msg), _("\n\nAlpine timed out (No user input for %d %s)\n"), to_hours,
 	    to_hours > 1 ? "hours" : "hour");
     msg[sizeof(msg)-1] = '\0';
     fast_clean_up();
@@ -301,7 +301,7 @@ term_signal(void)
     if(debugfile)
       fclose(debugfile);
 #endif 
-    printf(_("\n\nPine finished. Received terminate signal\n\n"));
+    printf(_("\n\nAlpine finished. Received terminate signal\n\n"));
 #endif	/* !DOS */
     exit(0);
 }
@@ -571,6 +571,7 @@ pipe_callback(PIPE_S *syspipe, int flags, void *data)
 	 * So we need to re-set it to child_signal and then set it back
 	 * when we're done.
 	 */
+	child_signalled = child_jump = 0;
 	syspipe->chld   = signal(SIGCHLD, child_signal);
 #endif
 #endif /* UNIX */
@@ -609,8 +610,6 @@ pipe_callback(PIPE_S *syspipe, int flags, void *data)
 	RETSIGTYPE (*alarm_sig)();
 	int	old_cue = F_ON(F_SHOW_DELAY_CUE, ps_global);
 
-	child_signalled = child_jump = 0;
-
 	/*
 	 * remember the current SIGALRM handler, and make sure it's
 	 * installed when we're finished just in case the longjmp
@@ -796,7 +795,7 @@ do_suspend(void)
 
     if(F_OFF(F_CAN_SUSPEND, pine)){
 	q_status_message(SM_ORDER | SM_DING, 3, 3,
-			 _("Pine suspension not enabled - see help text"));
+			 _("Alpine suspension not enabled - see help text"));
 	return(NO_OP_COMMAND);
     }
 
@@ -895,7 +894,7 @@ do_suspend(void)
 void
 suspend_notice(char *s)
 {
-    printf(_("\nPine suspended. Give the \"%s\" command to come back.\n"), s);
+    printf(_("\nAlpine suspended. Give the \"%s\" command to come back.\n"), s);
     fflush(stdout);
 }
 
@@ -905,7 +904,7 @@ suspend_notice(char *s)
 void
 suspend_warning(void)
 {
-    puts(_("Warning: Your IMAP connection will be closed if Pine"));
+    puts(_("Warning: Your IMAP connection will be closed if Alpine"));
     puts(_("is suspended for more than 30 minutes\n"));
     fflush(stdout);
 }
