@@ -23,7 +23,7 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	20 December 1989
- * Last Edited:	13 November 2006
+ * Last Edited:	20 December 2006
  */
 
 
@@ -719,7 +719,7 @@ long unix_ping (MAILSTREAM *stream)
       }
 				/* parse if mailbox changed */
       if ((LOCAL->ddirty || (sbuf.st_size != LOCAL->filesize)) &&
-	  unix_parse (stream,lock,LOCK_SH)) {
+	  unix_parse (stream,lock,LOCK_EX)) {
 				/* force checkpoint if double-dirty */
 	if (LOCAL->ddirty) unix_rewrite (stream,NIL,lock,NIL);
 				/* unlock mailbox */
@@ -1087,8 +1087,9 @@ long unix_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
   if (au && ret && tstream->uid_validity)
     (*au) (mailbox,tstream->uid_validity,dst);
   else mail_free_searchset (&dst);
-  unix_unlock (fd,NIL,lock);	/* unlock and close mailbox */
-  fclose (df);			/* note that unix_unlock() released the fd */
+  flock (fd,LOCK_UN);		/* unlock mailbox (can't use unix_unlock() */
+  if (lock && *lock) unlink (lock);
+  fclose (df);			/* close mailbox */
 				/* update last UID if we can */
   if (!tstream->rdonly) ((UNIXLOCAL *) tstream->local)->dirty = T;
   tstream = mail_close (tstream);
