@@ -1,10 +1,10 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: browse.c 257 2006-11-22 18:36:08Z hubert@u.washington.edu $";
+static char rcsid[] = "$Id: browse.c 380 2007-01-23 00:09:18Z hubert@u.washington.edu $";
 #endif
 
 /*
  * ========================================================================
- * Copyright 2006 University of Washington
+ * Copyright 2006-2007 University of Washington
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -197,7 +197,7 @@ static char *sa_BrowseHelpText[] = {
 N_("Help for Pilot (PIne's Looker-upper Of Things"),
 " ",
 N_("\tPilot is a simple, display-oriented file system browser based on the"),
-N_("~\tPine message system composer.  As with Pine, commands are displayed at"),
+N_("~\tAlpine message system composer.  As with Alpine, commands are displayed at"),
 N_("~\tthe bottom of the screen, and context-sensitive help is provided."),
 " ",
 N_("~\tPilot displays the current working directory at the top of the screen."),
@@ -233,11 +233,6 @@ N_("~\t~E\tEdit the selected file."),
 N_("~\t\tNote: Pilot invokes ~p~i~c~o, or the program defined by the ~E~D~I~T~O~R"),
 N_("~\t\t      environment variable, to edit the file."),
 " ",
-" ",
-N_("\tPine and Pico are trademarks of the University of Washington."),
-N_("\tNo commercial use of these trademarks may be made without prior"),
-N_("\twritten permission of the University of Washington."),
-" ",
 N_("End of Pilot Help."),
 NULL
 };
@@ -2700,7 +2695,7 @@ FileBrowse(char *dir, size_t dirlen, char *fn, size_t fnlen,
 	    char f[20000];
 	    size_t flen, dlen;
 
-	    f[0] = '\0';
+	    memset(f, 0, sizeof(f));
 
 	    rc = mswin_multopenfile(dir, dirlen, f, sizeof(f),
 		   (fb_flags & FB_ATTACH) ? NULL : "Text Files (*.txt)#*.txt");
