@@ -1,10 +1,10 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: main.c 200 2006-10-25 19:01:21Z hubert@u.washington.edu $";
+static char rcsid[] = "$Id: main.c 380 2007-01-23 00:09:18Z hubert@u.washington.edu $";
 #endif
 
 /*
  * ========================================================================
- * Copyright 2006 University of Washington
+ * Copyright 2006-2007 University of Washington
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -178,7 +178,7 @@ main(int argc, char *argv[])
 
     if(display_character_set)
       display_charmap = cpstr(display_character_set);
-#if   HAVE_LANGINFO_H
+#if   HAVE_LANGINFO_H && defined(CODESET)
     else
       display_charmap = cpstr(nl_langinfo(CODESET));
 #endif
@@ -193,7 +193,7 @@ main(int argc, char *argv[])
 
 
     if(use_system_translation){
-#if	(HAVE_WCHAR_H && HAVE_WCRTOMB && HAVE_WCWIDTH && HAVE_MBSTOWCS && !defined(_WINDOWS))
+#if	PREREQ_FOR_SYS_TRANSLATION
 	use_system++;
 	/* This modifies its arguments */
 	if(setup_for_input_output(use_system, &display_charmap, &keyboard_charmap,
@@ -339,8 +339,8 @@ main(int argc, char *argv[])
     curwp->w_flag |= WFMODE;		/* and force an update	*/
 
     if(timeoutset)
-      emlwrite(_("Checking for new mail every %D seconds"),
-	       (void *)get_input_timeout());
+      emlwrite(_("Checking for new mail every %s seconds"),
+	       comatose(get_input_timeout()));
 
 
     forwline(0, starton - 1);		/* move dot to specified line */
