@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: imap.c 237 2006-11-16 04:08:15Z mikes@u.washington.edu $";
+static char rcsid[] = "$Id: imap.c 394 2007-01-25 20:29:45Z hubert@u.washington.edu $";
 #endif
 
 /*
@@ -30,7 +30,6 @@ static char rcsid[] = "$Id: imap.c 237 2006-11-16 04:08:15Z mikes@u.washington.e
 #include "../pith/save.h"
 #include "../pith/util.h"
 #include "../pith/stream.h"
-///#include "../pith/osdep/debuging.h"
 
 
 /*
@@ -629,7 +628,7 @@ imap_referral(MAILSTREAM *stream, char *ref, long int code)
 
     if(ref && !struncmp(ref, "imap://", 7)){
 	char *folder = NULL;
-	long  uid_val, uid;
+	imapuid_t uid_val, uid;
 	int rv;
 
 	rv = url_imap_folder(ref, &folder, &uid, &uid_val, NULL, 1);
